/****************************************************************************/
/*              Beebem - (c) David Alan Gilbert 1994/1995                   */
/*              -----------------------------------------                   */
/* This program may be distributed freely within the following restrictions:*/
/*                                                                          */
/* 1) You may not charge for this program or for any part of it.            */
/* 2) This copyright message must be distributed with all copies.           */
/* 3) This program must be distributed complete with source code.  Binary   */
/*    only distribution is not permitted.                                   */
/* 4) The author offers no warrenties, or guarentees etc. - you use it at   */
/*    your own risk.  If it messes something up or destroys your computer   */
/*    thats YOUR problem.                                                   */
/* 5) You may use small sections of code from this program in your own      */
/*    applications - but you must acknowledge its use.  If you plan to use  */
/*    large sections then please ask the author.                            */
/*                                                                          */
/* If you do not agree with any of the above then please do not use this    */
/* program.                                                                 */
/* Please report any problems to the author at beebem@treblig.org           */
/****************************************************************************/
/* Beeb emulator - main file */



/*

DONE	Add SDL keyboard support + add additional keys to fake special keys via console.
DONE	Install new graphics code.
DONE	Fix line drawing so it's always within bounds.
DONE	Test video code for out of bounds.
DONE	Reduce screen size to target size.
DONE	Think of a way of reducing the frame rate to 25fps.
DONE	Add compiling support for GP2x.
DONE	Convert to faster rendering method.
DONE	Check that keyboard bug is not happening. (It was.)
DONE	Hack shift loading. (just make sure for first emulated second shift is down).
DONE	Investigate regulating timing (via frame change - beeb do time).
DONE	Port Critical's keyboard support.
DONE	add temp command line parser (which can load discs and keymappings).
DONE	Sound support (from 0.0.14).
DONE	Sound stops after n amount of time? I think this may be the cycle count flip? It was.
DONE	Make volume buttons work.
DONE	Keyboard associations as a drawing.
DONE	Add SET option support to kbd files.
DONE	Add half horizontal size support for teletext and graphics.
DONE	Graphics and Teletext should be flipable (modify the line drawing routines).
DONE	Add SET option to change CPU speed.
DONE	Add SET option to change volume.
DONE	Add SET option to change how the screen is centred.
DONE	MMSP2 + allocate some uncached upper memory for sound buffer.
DONE	Extend Critical's code to support all buttons with the execption of START.
DONE	Volume buttons only work if the GP2x volume button entries are set to GP2X_USEASVOLUME ("VOLUME" in config file).
DONE	Have two mapping buffers, 0 = normal, 1 = reversed. Other mapped keys are the upside down ones.
DONE	Allow a vertical screen flip and alternative button mapping via SET option.
DONE	Can discs be writable? Yes. Add SET option to make disks writable.

---------

ISSUE	Sound is running too fast. SDL seems to play mono sound at double speed (11Khz becomes 22Khz etc), have fudged for now.
ISSUE	Sound becomes distorted. Have moved the sequential sound buffer to uncached RAM, hopefully that will fix the problem.
ISSUE	at_exit isn't called for some reason. Doesn't seem to be called by the GP2x. Check compiler flags - wasn't compiler flags.

---------

*	Determine how GUI will look and create all graphics.



*	Two new configuration files, 1 is loaded prior to the .kbd file, the other after. That way you can set default options and force some.
	Some global settings like active_directory etc need to be defaulted here.
*	GUI from 0.0.14 with joystick support and with flipable graphics routines. New slide bar that only shows a troff, icon select (use this for file selector too), new font 6x8 as a graphic with outline.
*	Simple GUI interface like 0.0.14. Do I want tabs?
*	File selector.
*	On screen keyboard: Graphic selector (will need to control focus itself), is three graphics, normal, selected, focused. Is a list of
	areas and how they are associated for focus control. Can be used to implement a virtual keyboard. Can be part of an existing Surface.
*	On screen keyboard. Screen splits in two horizontally, the right hand 
	side for the keyboard. The bottom is the actual keyboard with the top
	being (eventually) a list of pre-programmed keys with descriptions 
	(useful for games like Elite that have lots of keys). Also re-mapping
	keys happens here too + the ability to list mapped keys etc.



*	Show free time (as a sum for the second) on the debug output.
*	Timer based shift loading (keyboard locked for first half second?).
*	Vertical and Horizontal centre when resolution changes. Investigate 
	repairing ScreenAdjust. Check for out of bounds writes.
*	Fix cursor so works correctly.



*	Move SDL initialization to sdl.cc
*	Add auto repeat on volume controls.
*	add a resources.cc file that uses getcwd. 
*	Convert to a C application.
*	Fix all warnings.










*	Parser for command line args (one from BeebEm 0.0.14).

*	Test save state - does it work at all?

*	See if you can cache previously generated fast-tables - Investigate a 
	hash table etc.

*	Harmonise the teletext code with 3.5.
*	New Teletext font (more like 3.5).

*	See if can slightly speed up teletext emulation.

*	Harmonise the 6502 core, VIAs and RAM access with BeebEm 3.5.

---------

GUI:

Load disc0, Load disc1

Sound off, sound on.

Map keys.

---------

*/


#include <signal.h>
#include <stdio.h>
#include <stdlib.h>

#include <iostream>

#include "6502core.h"
#include "beebmem.h"
#include "beebsound.h"
#include "sysvia.h"
#include "uservia.h"
#include "beebwin.h"
#include "disc8271.h"
#include "video.h"
#include "keyboard.h"

#include <stdio.h>

#include <SDL.h>
SDL_Surface *frame_buffer_p=NULL;

#include "via.h"
extern VIAState SysVIAState;
BeebWin *mainWin;

static char *Banner=
#include "header"
;

static char *Version="BeebEm for GP2x";

//#ifdef GP2X
///* GP2X button mapping:
// */
//
//#	define GP2X_BUTTON_UP              (0)
//#	define GP2X_BUTTON_DOWN            (4)
//#	define GP2X_BUTTON_LEFT            (2)
//#	define GP2X_BUTTON_RIGHT           (6)
//#	define GP2X_BUTTON_UPLEFT          (1)
//#	define GP2X_BUTTON_UPRIGHT         (7)
//#	define GP2X_BUTTON_DOWNLEFT        (3)
//#	define GP2X_BUTTON_DOWNRIGHT       (5)
//#	define GP2X_BUTTON_CLICK           (18)
//#	define GP2X_BUTTON_A               (12)
//#	define GP2X_BUTTON_B               (13)
//#	define GP2X_BUTTON_X               (14)
//#	define GP2X_BUTTON_Y               (15)
//#	define GP2X_BUTTON_L               (10)
//#	define GP2X_BUTTON_R               (11)
//#	define GP2X_BUTTON_START           (8)
//#	define GP2X_BUTTON_SELECT          (9)
//#	define GP2X_BUTTON_VOLUP           (16)
//#	define GP2X_BUTTON_VOLDOWN         (17)
//
//static struct BeebKeyTrans GP2XtoBeebEmKeymap[]={
// GP2X                 BBC                BBC KEY NAME
//{GP2X_BUTTON_Y,		6,2},		// SPACE
//{GP2X_BUTTON_A,		4,9},		// RETURN
//{GP2X_BUTTON_LEFT,	4,0},		// CAPS LOCK
//{GP2X_BUTTON_UPLEFT,	4,0},		// CAPS LOCK
//{GP2X_BUTTON_DOWNLEFT,	4,0},		// CAPS LOCK
//{GP2X_BUTTON_RIGHT,	0,1},		// CONTROL
//{GP2X_BUTTON_UPRIGHT,	0,1},		// CONTROL
//{GP2X_BUTTON_DOWNRIGHT,	0,1},		// CONTROL
//
//{-1, -1, -1}              // ** END OF LIST **
//};
//
//int ConvertGP2XButtonToBBCKey(int button, int *col, int *row)
//{
//        struct BeebKeyTrans *p = GP2XtoBeebEmKeymap;
//
//        for(;((p->row != -1) && (p->sym != button));p++) {}
//
//        *(row) = p->row;
//        *(col) = p->col;
//
//        return *row<0 ? 0 : 1;
//}
//#endif



void at_exit(void)
{
	printf("running at_exit.\n");
	GP2x_FreeResources();
	SDL_Quit();

#ifdef GP2X
	OSS_Mixer_Quit();
#endif

#ifdef GP2X
#	ifdef PRODUCTION
		printf("loading GP2X menu.\n");
		chdir("/usr/gp2x");
		execl("/usr/gp2x/gp2xmenu", "/usr/gp2x/gp2xmenu", NULL);
#	endif
#endif
}

/* The GP2x doesn't seem to call atexit after calling exit()?
 *
 * I'll just fudge it for now...
 */
#ifdef GP2X
	void do_exit(int r) { at_exit(); exit(r); }
#       define EXIT do_exit
#else
#       define EXIT exit
#endif

void cls(void)
{
	SDL_FillRect(frame_buffer_p, NULL, 0);
}


int main(int argc,char *argv[]) {
	char mapping[256], *file_type_p, *filename_p;
	int flags;
#ifdef GP2X
	SDL_Joystick *joystick_p = NULL;
#endif

	/* Print start message on console:
	 */
	std::cout << Banner << "\n" << "Version: " << Version << "\n";

	/* Initialize SDL:
	 */

	flags = SDL_INIT_VIDEO /* | SDL_INIT_AUDIO */ ;
#ifdef GP2X
	flags |= SDL_INIT_JOYSTICK;
#endif
	if (SDL_Init(flags) <0) {
		fprintf(stderr, "Unable to initialise SDL: %s\n"
		 , SDL_GetError());
		return false;
	}

#ifndef GP2X
	/* Cleanup SDL when exiting (and if GP2x load menu).
	 */
	atexit(at_exit);
#endif

	/* Setup a frame buffer to play with.
	 */
	flags = SDL_HWSURFACE | SDL_HWPALETTE;
#ifndef GP2X
	if ( (frame_buffer_p=SDL_SetVideoMode(800, 600, 8, flags)) == NULL ) {
#else
	if ( (frame_buffer_p=SDL_SetVideoMode(320, 240, 8, flags)) == NULL ) {
#endif
		fprintf(stderr, "Unable to set video mode: %s\n"
		 , SDL_GetError());
		EXIT(1);
	}
#ifdef GP2X
	SDL_ShowCursor(SDL_DISABLE);
#endif

#ifdef GP2X
	if ( (joystick_p=SDL_JoystickOpen(0) ) == NULL) {
		fprintf(stderr, "Couldn't open the joystick: %s\n"
		 , SDL_GetError());
		EXIT(1);
	}
#endif

#ifndef GP2X
	/* Fill screen with some boxes to denote boundries.
	 */
	SDL_FillRect(frame_buffer_p, NULL, SDL_MapRGB(frame_buffer_p->format
	 , 0xff, 0x00, 0x00));
	SDL_Rect rect= {0, 0, 322, 258};
	Uint32 color = SDL_MapRGB(frame_buffer_p->format, 0xff, 0xff, 0x00);
	SDL_FillRect(frame_buffer_p, &rect, color);
	SDL_UpdateRect(frame_buffer_p, 0, 0, frame_buffer_p->w
	 , frame_buffer_p->h);
	
	rect= (SDL_Rect) {0, 0, 322, 242};
	color = SDL_MapRGB(frame_buffer_p->format, 0x00, 0xff, 0xff);
	SDL_FillRect(frame_buffer_p, &rect, color);
	SDL_UpdateRect(frame_buffer_p, 0,0,frame_buffer_p->w,frame_buffer_p->h);
#endif


#ifdef GP2X
	/* Setup volume on GP2x:
	 */
	OSS_Mixer_Init();
	OSS_Mixer_SetVolume(0);
	printf("Volume set to %d.\n", OSS_Mixer_GetVolume());
#endif


	/* Initialize the GP2x hardware:
	 */
	GP2x_Initialize();

	/* Initialize abstracted host environment.
	 */
	// [TODO] Get rid of this, use normal C calls to everything in here, everything is hardwired to mainWin anyway.
	mainWin=new BeebWin();

	/* Initialize emulator core:
	 */
	BeebMemInit();
  	Init6502core();
 	SysVIAReset();
#ifdef SOUNDSUPPORT
  	SoundInit();
#endif
  	UserVIAReset();
  	VideoInit();
  	Disc8271_reset();

#ifdef GP2X
	GP2x_SetCPUSpeed(200);
#endif

	/* Load disc hack (for now):
	 */

	// [TODO] Parser for commands (look at BeebEm 3.5).

	if (argc>1) {
		filename_p = argv[1];

		if ( (file_type_p=strrchr(filename_p, (int) '.')) == NULL) {
			printf("Supplied disc image filename does not have a file extension.\n");
			EXIT(1);
		}
	
		printf("Loading disc image '%s'\n", filename_p);

		if (strcasecmp(file_type_p, ".ssd") == 0) 
			LoadSimpleDiscImage(filename_p, 0, 0, 80);
		else if (strcasecmp(file_type_p, ".dsd") == 0)
			LoadSimpleDSDiscImage(filename_p, 0, 80);
		else {
			printf("Unsupported disc image.\n");
			EXIT(1);
		}

		if (argc == 2) {
			strncpy(mapping, filename_p, strlen(filename_p)-strlen(file_type_p));
			mapping[strlen(filename_p)-strlen(file_type_p)]='\0';
			strcat(mapping, ".kbd");
		} else {
			strcpy(mapping, argv[2]);
		}

		printf("Loading button mapping '%s'\n", mapping);

		kbd_load_mapping(mapping);
	} else {
		printf("No disc image specified\n");
	}



	/* Main loop:
	 */

	/* Shift boot the GP2X for now
	 */
	// [TODO] Make this time based and dump key presses within under the first half second.
	mainWin->ShiftBooted = true;
	BeebKeyDown(0, 0);

	int done, beeb_key_column, beeb_key_row, col, row;
	done = beeb_key_column = beeb_key_row = col = row = 0;
	do {

		SDL_Event event;
		while (SDL_PollEvent(&event)) {
			switch (event.type) {
				
				case SDL_QUIT:
					done=1;
					break;

			/* Handle SDL Keyboard:
			 */
				case SDL_KEYDOWN:

					/* Debugging hacks: -----------------------------------------------
					 */

					if (event.key.keysym.sym == SDLK_F1) {
						if (mainWin->OnScreenKeyboardShown())
							mainWin->HideOnScreenKeyboard();
						else
							mainWin->ShowOnScreenKeyboard();

						cls();
						break;
					}

					if (event.key.keysym.sym == SDLK_F2) {
						if (mainWin->ScreenIsReversed())
							mainWin->NormaliseScreenOrientation();
						else
							mainWin->ReverseScreenOrientation();
						cls();
						break;
					}



					/* ----------------------------------------------------------------
					 */


					/* Handle shift booting:
					 */
					if (mainWin->ShiftBooted) {
						mainWin->ShiftBooted = false;
						BeebKeyUp(0, 0);
					}
					if ( ConvertSDLKeyToBBCKey( event.key.keysym.sym, &beeb_key_column, &beeb_key_row) )
						BeebKeyDown(beeb_key_row, beeb_key_column);
					else if (beeb_key_row == -2) {
						/* Handle reset */
						Init6502core();
						Disc8271_reset();
					}
					break;

				case SDL_KEYUP:
					if ( ConvertSDLKeyToBBCKey(event.key.keysym.sym, &beeb_key_column, &beeb_key_row) )
						BeebKeyUp(beeb_key_row, beeb_key_column);
					break;

			/* Handle GP2x buttons:
	 	 	 */
#ifdef GP2X
                        case SDL_JOYBUTTONDOWN:
					if (event.jbutton.button == GP2X_BUTTON_VOLUP) {
						if (gp2xButtonToSDLK(event.jbutton.button) == KEY_USEASVOLUME) 
							if (OSS_Mixer_GetVolume()<100)
								OSS_Mixer_SetVolume(OSS_Mixer_GetVolume()+10);
					} else if (event.jbutton.button == GP2X_BUTTON_VOLDOWN) {
						if (gp2xButtonToSDLK(event.jbutton.button) == KEY_USEASVOLUME)
							if (OSS_Mixer_GetVolume()>0)
								OSS_Mixer_SetVolume(OSS_Mixer_GetVolume()-10);
					}


					/* Debugging hacks: -----------------------------------------------
					 */

/*					// [TODO] Make GUI do this:
					if (event.jbutton.button == GP2X_BUTTON_L) {
						if (mainWin->OnScreenKeyboardShown())
							mainWin->HideOnScreenKeyboard();
						else
							mainWin->ShowOnScreenKeyboard();

						cls();
					}

					if (event.jbutton.button == GP2X_BUTTON_R) {
						if (mainWin->ScreenIsReversed()) {
							AllDirectionButtonsUp();
							mainWin->NormaliseScreenOrientation();
						} else {
							AllDirectionButtonsUp();
							mainWin->ReverseScreenOrientation();
						}
						cls();
					}
*/

					/* ----------------------------------------------------------------
					 */




					// There should be no 'break;' here, I want it to drop through.

                        case SDL_JOYBUTTONUP:
                                {

                                        // Reset shift state if it was set by Run Disc
					// [TODO] Make this timer based.
                                        if (mainWin->ShiftBooted)
                                        {
                                                mainWin->ShiftBooted = false;
                                                BeebKeyUp(0, 0);
                                        }

                                        int pressed=0, ordinalHit=0;
                                        //SDL_keysym keysym;
                                        int keysym = 0;
                                        //keysym.sym = SDLK_UNKNOWN;

                                        if (event.jbutton.button == GP2X_BUTTON_START)
                                        {
						// [TODO] Menu.
						done=1;
						break;
                                        }
                                        else
                                        {
                                                if (	event.jbutton.button == GP2X_BUTTON_LEFT ||
                                                        event.jbutton.button == GP2X_BUTTON_RIGHT ||
                                                        event.jbutton.button == GP2X_BUTTON_UP ||
                                                        event.jbutton.button == GP2X_BUTTON_DOWN)
                                                {
                                                        //printf("hit ordinal\n");
                                                        keysym = gp2xButtonToSDLK(event.jbutton.button);
                                                        //AllDirectionButtonsUp();
                                                        ordinalHit = 1;
                                                        HandleButtonPressForKeysym(event.type == SDL_JOYBUTTONDOWN, keysym);
                                                }
                                                else if (event.jbutton.button == GP2X_BUTTON_A ||
                                                        event.jbutton.button == GP2X_BUTTON_B ||
                                                        event.jbutton.button == GP2X_BUTTON_X ||
                                                        event.jbutton.button == GP2X_BUTTON_Y ||

							event.jbutton.button == GP2X_BUTTON_CLICK ||
							event.jbutton.button == GP2X_BUTTON_L ||
							event.jbutton.button == GP2X_BUTTON_R ||
							event.jbutton.button == GP2X_BUTTON_VOLUP ||
							event.jbutton.button == GP2X_BUTTON_VOLDOWN ||

                                                        event.jbutton.button == GP2X_BUTTON_SELECT)
                                                {
                                                        keysym = gp2xButtonToSDLK(event.jbutton.button);
                                                        HandleButtonPressForKeysym(event.type == SDL_JOYBUTTONDOWN, keysym);
                                                }

                                                if (event.jbutton.button == GP2X_BUTTON_UPLEFT)
                                                {
                                                        //printf("hit upleft\n");

                                                        if (!ordinalHit)
                                                                AllDirectionButtonsUp();

                                                        if (SDL_JoystickGetButton(joystick_p, GP2X_BUTTON_LEFT))
                                                        {
                                                                // weight it towards left
								if (! mainWin->ScreenIsReversed()) {
                                                                	HandleButtonPress(1, JOY_LEFT);
                                                                	HandleButtonPress(0, JOY_UP);
								} else {
									HandleButtonPress(1, R_JOY_LEFT);
									HandleButtonPress(0, R_JOY_UP);
								}
                                                                //printf("weighted left\n");
                                                        }
                                                        else if (SDL_JoystickGetButton(joystick_p, GP2X_BUTTON_UP))
                                                        {
                                                                // weight it towards up
								if (! mainWin->ScreenIsReversed()) {
                                                                	HandleButtonPress(1, JOY_UP);
                                                                	HandleButtonPress(0, JOY_LEFT);
								} else {
									HandleButtonPress(1, R_JOY_UP);
									HandleButtonPress(0, R_JOY_LEFT);
								}
                                                                //printf("weighted up\n");
                                                        }
                                                        else
                                                        {
                                                                // do both
								if (! mainWin->ScreenIsReversed()) {
                                                                	HandleButtonPress(event.type == SDL_JOYBUTTONDOWN, JOY_UP);
                                                                	HandleButtonPress(event.type == SDL_JOYBUTTONDOWN, JOY_LEFT);
								} else {
									HandleButtonPress(event.type == SDL_JOYBUTTONDOWN, R_JOY_UP);
									HandleButtonPress(event.type == SDL_JOYBUTTONDOWN, R_JOY_LEFT);
								}
                                                        }
                                                }
                                                else if (event.jbutton.button == GP2X_BUTTON_UPRIGHT)
                                                {
                                                        //printf("hit upright\n");
                                                        if (!ordinalHit)
                                                                AllDirectionButtonsUp();

                                                        if (SDL_JoystickGetButton(joystick_p, GP2X_BUTTON_RIGHT))
                                                        {
                                                                // weight it towards right
								if (! mainWin->ScreenIsReversed()) {
									HandleButtonPress(1, JOY_RIGHT);
                                                                	HandleButtonPress(0, JOY_UP);
								} else {
									HandleButtonPress(1, R_JOY_RIGHT);
									HandleButtonPress(0, R_JOY_UP);
								}
                                                                //printf("weighted right\n");
                                                        }
                                                        else if (SDL_JoystickGetButton(joystick_p, GP2X_BUTTON_UP))
                                                        {
                                                                // weight it towards up
								if (! mainWin->ScreenIsReversed()) {
                                                                	HandleButtonPress(1, JOY_UP);
                                                                	HandleButtonPress(0, JOY_RIGHT);
								} else {
									HandleButtonPress(1, R_JOY_UP);
									HandleButtonPress(0, R_JOY_RIGHT);
								}
                                                                //printf("weighted up\n");
                                                        }
                                                        else
                                                        {
								if (! mainWin->ScreenIsReversed()) {
                                                                	HandleButtonPress(event.type == SDL_JOYBUTTONDOWN, JOY_UP);
                                                                	HandleButtonPress(event.type == SDL_JOYBUTTONDOWN, JOY_RIGHT);
								} else {
									HandleButtonPress(event.type == SDL_JOYBUTTONDOWN, R_JOY_UP);
									HandleButtonPress(event.type == SDL_JOYBUTTONDOWN, R_JOY_RIGHT);
								}
                                                        }
                                                }
                                                else if (event.jbutton.button == GP2X_BUTTON_DOWNLEFT)
                                                {
                                                        //printf("hit downleft\n");
                                                        if (!ordinalHit)
                                                                AllDirectionButtonsUp();

                                                        if (SDL_JoystickGetButton(joystick_p, GP2X_BUTTON_LEFT))
                                                        {
                                                                // weight it towards left
								if (! mainWin->ScreenIsReversed()) {
                                                                	HandleButtonPress(1, JOY_LEFT);
                                                                	HandleButtonPress(0, JOY_DOWN);
								} else {
									HandleButtonPress(1, R_JOY_LEFT);
									HandleButtonPress(0, R_JOY_DOWN);
								}
                                                                //printf("weighted left\n");
                                                        }
                                                        else if (SDL_JoystickGetButton(joystick_p, GP2X_BUTTON_DOWN))
                                                        {
                                                                // weight it towards down
								if (! mainWin->ScreenIsReversed()) {
                                                                	HandleButtonPress(1, JOY_DOWN);
                                                                	HandleButtonPress(0, JOY_LEFT);
								} else {
									HandleButtonPress(1, R_JOY_DOWN);
									HandleButtonPress(0, R_JOY_LEFT);
								}
                                                                //printf("weighted down\n");
                                                        }
                                                        else
                                                        {
								if (! mainWin->ScreenIsReversed()) {
                                                                	HandleButtonPress(event.type == SDL_JOYBUTTONDOWN, JOY_DOWN);
                                                                	HandleButtonPress(event.type == SDL_JOYBUTTONDOWN, JOY_LEFT);
								} else {
									HandleButtonPress(event.type == SDL_JOYBUTTONDOWN, R_JOY_DOWN);
									HandleButtonPress(event.type == SDL_JOYBUTTONDOWN, R_JOY_LEFT);
								}
                                                        }

                                                }
                                                else if (event.jbutton.button == GP2X_BUTTON_DOWNRIGHT)
                                                {
                                                        //printf("hit downright\n");

                                                        if (!ordinalHit)
                                                                AllDirectionButtonsUp();

                                                        if (SDL_JoystickGetButton(joystick_p, GP2X_BUTTON_RIGHT))
                                                        {
                                                                // weight it towards right
								if (! mainWin->ScreenIsReversed()) {
                                                                	HandleButtonPress(event.type == SDL_JOYBUTTONDOWN, JOY_RIGHT);
                                                                	HandleButtonPress(0, JOY_DOWN);
								} else {
									HandleButtonPress(event.type == SDL_JOYBUTTONDOWN, R_JOY_RIGHT);
									HandleButtonPress(0, R_JOY_DOWN);
								}
                                                                //printf("weighted right\n");
                                                        }
                                                        else if (SDL_JoystickGetButton(joystick_p, GP2X_BUTTON_DOWN))
                                                        {
                                                                // weight it towards down
								if (! mainWin->ScreenIsReversed()) {
                                                                	HandleButtonPress(event.type == SDL_JOYBUTTONDOWN, JOY_DOWN);
                                                                	HandleButtonPress(0, JOY_RIGHT);
								} else {
									HandleButtonPress(event.type == SDL_JOYBUTTONDOWN, R_JOY_DOWN);
									HandleButtonPress(0, R_JOY_RIGHT);
								}
                                                                //printf("weighted down\n");
                                                        }
                                                        else
                                                        {
								if (! mainWin->ScreenIsReversed()) {
                                                                	HandleButtonPress(event.type == SDL_JOYBUTTONDOWN, JOY_DOWN);
                                                                	HandleButtonPress(event.type == SDL_JOYBUTTONDOWN, JOY_RIGHT);
								} else {
									HandleButtonPress(event.type == SDL_JOYBUTTONDOWN, R_JOY_DOWN);
									HandleButtonPress(event.type == SDL_JOYBUTTONDOWN, R_JOY_RIGHT);
								}
                                                        }
                                                }

                                        }

                                        if (keysym != 0)
                                        {
                                                if(row==-2)
                                                {
	                                              Init6502core();
        	                                      Disc8271_reset();
                                                }


                                        }
                                }
                                break;
#endif

			}
		}

		/* Actually will exec. approx. half a frame of cycles (good
		 * enough hack for now).
		 */
		Exec6502Instruction();

	} while (! done);

	EXIT(0);
}
